#pragma once

#include <cmath>
#include <iostream>
#include <complex>


namespace resumm {
  typedef std::complex<double> dcomplex;
  const dcomplex I(0.0,1.0);
  inline double ABS (const dcomplex & a) { return abs(a); }
}

/*
namespace resumm {
  class dcomplex;
}

class resumm::dcomplex{
private:
  double r, i;

public:
  inline dcomplex(double re=0.0, double im= 0.0): r(re), i(im) {}
  inline dcomplex(const dcomplex & a): r(a.r), i(a.i){}

  inline double re() { return r; }
  inline double im() { return i; }
  inline double norm() { return (r*r + i*i); }
  inline double arg () { return atan2(i,r); }  //   should be in the range  -PI to PI 
  inline double abs () { return sqrt(r*r + i*i); }

  inline dcomplex & operator = (const dcomplex & a){
    if (r != a.r || i != a.i) {
      r = a.r; i = a.i;
    }
    return *this;
  }
  inline dcomplex  operator - () const {dcomplex A = *this; A.r = -r; 
    A.i = -i; return A;
  }
  inline dcomplex& operator += (double a){ r += a; return *this;}
  inline dcomplex& operator -= (double a){ r -= a; return *this;}
  inline dcomplex& operator *= (double a){ r *= a; i *= a; return *this;}
  inline dcomplex& operator /= (double a){ r /= a; i /= a; return *this;}
  inline dcomplex& operator += (const dcomplex &a) {
    r +=a.r ; i += a.i; return *this;}
  inline dcomplex& operator -= (const dcomplex &a) {
    r -=a.r; i -= a.i; return *this;}
  inline dcomplex& operator *= (const dcomplex &a) { double rr = r;
    r = rr*a.r - i*a.i; i = i*a.r + rr*a.i; return *this;}
  inline dcomplex& operator /= (const dcomplex &a){
    double m = a.norm(); double rr = r;
    r = (rr * a.r+ i*a.i)/m; i = (i*a.r - rr*a.i)/m; return *this;}
  inline dcomplex& operator ++ () {r += 1.0; return *this;}
  inline dcomplex& operator -- () {r -= 1.0; return *this;}

  / *
  friend std::istream& operator >> (std::istream& is, dcomplex& a);
  friend std::ostream& operator << (std::ostream& os,  const dcomplex& a);

  friend inline dcomplex operator + (const dcomplex &a, const dcomplex &b) { return dcomplex(a.r+b.r,a.i+b.i); }
  friend inline dcomplex operator + (const dcomplex &a, double b)          { return dcomplex(a.r + b, a.i); }
  friend inline dcomplex operator + (double b, const dcomplex &a)          { return dcomplex(a.r + b, a.i); }
  friend inline dcomplex operator - (const dcomplex &a, const dcomplex &b) { return dcomplex(a.r-b.r,a.i-b.i); }
  friend inline dcomplex operator - (const dcomplex &a, double b)          { return dcomplex(a.r - b, a.i); }
  friend inline dcomplex operator - (double b, const dcomplex &a)          { return dcomplex(b - a.r, -a.i); }
  friend inline dcomplex operator * (const dcomplex &a, const dcomplex &b) { return dcomplex(a.r*b.r-a.i*b.i,a.i*b.r + a.r*b.i); }
  friend inline dcomplex operator * (const dcomplex &a, double b)          { return dcomplex(a.r* b, a.i*b); }
  friend inline dcomplex operator * (double b, const dcomplex &a)          { return dcomplex(a.r* b, a.i*b); }
  friend inline dcomplex operator / (const dcomplex &a, const dcomplex &b) {
    double m = norm(b);
    return dcomplex((a.r*b.r+a.i*b.i)/m,(a.i*b.r- a.r*b.i)/m);
  }
  friend inline dcomplex operator / (const dcomplex &a, double b) { return resumm::dcomplex(a.r/b, a.i/b); }
  friend inline dcomplex operator / (double b, const dcomplex &a) {
    double m = norm(a);
    return dcomplex(b *a.r/m, -b*a.i/m);
  }
  friend inline int operator == (const dcomplex &a, const dcomplex &b) { return((a.r == b.r) && (a.i == b.i) ? 1 : 0 );}
  friend inline int operator != (const dcomplex &a, const dcomplex &b) { return((a.r == b.r) && (a.i == b.i) ? 0 : 1);}
  friend inline dcomplex conj(const dcomplex & a) { return dcomplex(a.r,-a.i); }
  friend inline dcomplex log(const dcomplex &a) { return dcomplex( 0.5 * log(norm(a)), arg(a)); }
  friend inline dcomplex cos(const dcomplex &a) { return dcomplex( cos(a.r)*cosh(a.i), sin(a.r)*sinh(a.i)); }
  friend inline dcomplex sin(const dcomplex &a) { return dcomplex( sin(a.r)*cosh(a.i), cos(a.r)*sinh(a.i)); }
  * /
};
const resumm::dcomplex I(0.0,1.0);


inline double real(const resumm::dcomplex & a) { return a.re();}
inline double imag(const resumm::dcomplex & a) { return a.im();}
inline double norm(const resumm::dcomplex & a) { return a.norm();}
inline double arg (const resumm::dcomplex & a) { return a.arg(); } //   should be in the range  -PI to PI 
inline double ABS (const resumm::dcomplex & a) { return a.abs(); }
inline double abs (const resumm::dcomplex & a) { return a.abs(); }



inline resumm::dcomplex polar(double m,double ar) { return  resumm::dcomplex(m*cos(ar),m*sin(ar));}

inline resumm::dcomplex pow(double a,const resumm::dcomplex & b){
   return exp(log(a)*b);}

inline resumm::dcomplex pow(const resumm::dcomplex & a,double b){
   return exp(log(a)*b);}

inline resumm::dcomplex pow(const resumm::dcomplex & a,const resumm::dcomplex & b){
   return exp(log(a)*b);}

inline resumm::dcomplex sqrt(const resumm::dcomplex & a) {
  double m = sqrt(sqrt(norm(a)));
  return polar(m,0.5*arg(a));
}

inline resumm::dcomplex exp(const resumm::dcomplex & a) {
  double m = exp(real(a));
  return polar(m,imag(a));
}



*/
